import React, { useReducer, useRef, useMemo, useEffect, useContext, useLayoutEffect, createContext } from 'react';
import { runSaga, stdChannel } from 'redux-saga';
const SagaContext = createContext({});
export const SagaProvider = (props) => {
    return React.createElement(SagaContext.Provider, Object.assign({}, props));
};
const useIsomorphicLayoutEffect = typeof window !== 'undefined' ? useLayoutEffect : useEffect;
export function useSagaReducer(saga, reducer, initializerArg, initializer, runSagaOptions) {
    const [state, reactDispatch] = useReducer(reducer, initializerArg, initializer);
    const stateRef = useRef(state);
    useIsomorphicLayoutEffect(() => {
        stateRef.current = state;
    }, [state]);
    const sagaIO = useMemo(() => {
        const channel = stdChannel();
        const dispatch = (action) => {
            setImmediate(channel.put, action);
            reactDispatch(action);
        };
        const getState = () => stateRef.current;
        return {
            channel,
            dispatch,
            getState
        };
    }, []);
    const sagaContextValue = useContext(SagaContext);
    useEffect(() => {
        const options = runSagaOptions || {};
        const context = {
            ...sagaContextValue,
            ...options.context
        };
        const sagaOptions = {
            ...sagaIO,
            context,
            sagaMonitor: options.sagaMonitor,
            onError: options.onError,
            effectMiddlewares: options.effectMiddlewares
        };
        const task = runSaga(sagaOptions, saga);
        return () => {
            task.cancel();
        };
    }, []);
    return [state, sagaIO.dispatch];
}
export default useSagaReducer;
